#include <sta/tacos/system/events.hpp>
#include <sta/rtos/event.hpp>
#include <sta/rtos/timer.hpp>

#include <sta/debug/assert.hpp>

#include <cmsis_os2.h>
#include <FreeRTOS.h>


namespace
{
	// Event handle
	sta::RtosEvent * systemEvents = nullptr;

	uint8_t tickCounter = 0;

	// Tick timer handle
	sta::RtosTimer * tickTimer = nullptr;
}


namespace sta
{
	namespace tacos
	{
		namespace events
		{
			void init()
			{
				if (systemEvents == nullptr)
				{
					systemEvents = new sta::RtosEvent();
				}

				if (tickTimer == nullptr)
				{
					tickTimer = new sta::RtosTimer([]() {
						uint32_t flags = 0x00;

						if (tickCounter % 2 == 0)
						{
							flags |= Types::TICK_100Hz;

							if ((tickCounter / 2) % 2 == 0) {
								flags |= Types::TICK_50Hz;
							}
							else
							{
								flags |= Types::TOCK_50Hz;
							}

							if (tickCounter % 10 == 0 && ((tickCounter / 10) % 2) == 0)
							{
								flags |= Types::TICK_10Hz;
							}
							else if (tickCounter % 10 == 0 && (tickCounter / 10) == 1) {
								flags |= Types::TOCK_10Hz;
							}

							if (tickCounter % 100 == 0 && (tickCounter / 100) % 2 == 0) {
								flags |= Types::TICK_1Hz;
							}
							else if (tickCounter % 100 == 0 && (tickCounter / 100) == 1) {
								flags |= Types::TOCK_1Hz;
							}
						}
						else
						{
							flags |= Types::TOCK_100Hz;

							if (tickCounter % 5 == 0 && ((tickCounter / 5) % 2) == 0)
							{
								flags |= Types::TICK_20Hz;
							}
							else if (tickCounter % 5 == 0 && (tickCounter / 5) == 1) {
								flags |= Types::TOCK_20Hz;
							}
						}

						tickCounter = ++tickCounter % 200;

						signal(flags);
					}, true);

					tickTimer->start(10);
				}
			}

			void signal(uint32_t flags)
			{
				STA_ASSERT_MSG(systemEvents != nullptr, "System events not initialized");
				systemEvents->set(flags);
			}

			void wait(uint32_t flags, uint32_t options, uint32_t timeout)
			{
				STA_ASSERT_MSG(systemEvents != nullptr, "System events not initialized");
				systemEvents->peek(flags, timeout);
			}
	
			void signalStartup()
			{
				signalSystemEvents(STA_TACOS_SYSTEM_EVENTS_STARTUP);
			}

			void waitForStartup()
			{
				waitForSystemEvents(STA_TACOS_SYSTEM_EVENTS_STARTUP, osFlagsWaitAll, osWaitForever);
			}
		}  // namespace events
	} // namespace tacos
} // namespace sta
